if __name__ == '__main__':
  #Used to guarantee to use at least Wx2.8
  import wxversion
  wxversion.ensureMinimal('2.8')
import wx
import matplotlib as mpl
if __name__ == '__main__':
  mpl.use('WXAgg') #or mpl.use('WX')

from matplotlib.backends.backend_wxagg import NavigationToolbar2WxAgg as NavigationToolbar
import time,os

class Path():
  @staticmethod
  def GetImage():
    path=__file__
    try:symPath=os.readlink(path) #follow symbolic link
    except (AttributeError,OSError) as e:pass
    else:
      path=symPath
    path=os.path.abspath(path)
    path=os.path.dirname(path)
    return os.path.join(path,'images')

class StopWatch():
  @classmethod
  def Start(cls):
    cls.ts=time.time()
  @classmethod
  def Log(cls,str=None,restart=True):
    ts=time.time()
    print '%.6f'%(ts-cls.ts),str
    if restart:
      cls.ts=ts

class SliderGroup():
  def __init__(self, parent, label, range=(0,100),val=0):
    self.sliderLabel = wx.StaticText(parent, label=label)
    self.sliderText = wx.TextCtrl(parent, -1, style=wx.TE_PROCESS_ENTER)
    self.slider = wx.Slider(parent, -1)
    self.slider.SetRange(range[0],range[1])      
    sizer = wx.BoxSizer(wx.HORIZONTAL)
    sizer.Add(self.sliderLabel, 0, wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, border=2)
    sizer.Add(self.sliderText, 0, wx.EXPAND | wx.ALIGN_CENTER | wx.ALL, border=2)
    sizer.Add(self.slider, 1, wx.EXPAND)
    self.sizer = sizer
    self.slider.Bind(wx.EVT_SLIDER, self.sliderHandler)
    self.sliderText.Bind(wx.EVT_TEXT_ENTER, self.sliderTextHandler)
    self.SetValue(val)

  def SetValue(self, value):
    self.value = value
    self.slider.SetValue(value)
    self.sliderText.SetValue(str(value))
  
  def SetCallback(self,funcCB,usrData):
    self.cbFuncData=(funcCB,usrData)

  def Callback(self,value,msg):
    try:
      (funcCB,usrData)=self.cbFuncData
    except BaseException as e:
      pass
    else:
      funcCB(usrData,value,msg)

  def sliderHandler(self, evt):
    value = evt.GetInt()
    self.sliderText.SetValue(str(value))
    self.value=value
    self.Callback(value,0)      
      
  def sliderTextHandler(self, evt):
    value = int(self.sliderText.GetValue())
    self.slider.SetValue(value)
    value = self.slider.Value
    self.sliderText.SetValue(str(value))
    self.value=value
    self.Callback(value,0)      

def GetSlice(idxXY,shp,wxAxCtrlLst):
  '''returns a slice list to select data'''
  sl=[None]*len(shp)
  for ax in wxAxCtrlLst:
    sl[ax.idx]=ax.value
  for i in idxXY:
    sl[i]=slice(None)
  sl=tuple(sl)
  return sl


def AddToolbar(parent,sizer):
  toolbar = NavigationToolbar(parent)
  toolbar.Realize()
  if wx.Platform == '__WXMAC__':
    # Mac platform (OSX 10.3, MacPython) does not seem to cope with
    # having a toolbar in a sizer. This work-around gets the buttons
    # back, but at the expense of having the toolbar at the top
    parent.SetToolBar(toolbar)
  else:
    # On Windows platform, default window size is incorrect, so set
    # toolbar width to figure width.
    tw, th = toolbar.GetSizeTuple()
    fw, fh = parent.GetSizeTuple()
    # By adding toolbar in sizer, we are able to put it at the bottom
    # of the frame - so appearance is closer to GTK version.
    # As noted above, doesn't work for Mac.
    toolbar.SetSize(wx.Size(fw, th))
    sizer.Add(toolbar, 0, wx.LEFT | wx.EXPAND)
  # update the axes menu on the toolbar
  toolbar.update()
  return toolbar




#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
implements an image view to show a colored image of a hdf5 dataset.
'''

if __name__ == '__main__':
  #Used to guarantee to use at least Wx2.8
  import wxversion
  wxversion.ensureMinimal('2.8')
import wx
import matplotlib as mpl
if __name__ == '__main__':
  mpl.use('WXAgg')
  #or mpl.use('WX')
  #matplotlib.get_backend()

import os,h5py
import numpy as np
import utilities as ut
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
import os,h5py
from GLCanvasImg import *
from libDetXR.procRoiStat import ProcRoiStat

from hdfImageGL import HdfImageGLFrame
from glumpy.image.texture import Texture
from scipy import ndimage as ndi

###########################################

class ProcRoiStatFrame(HdfImageGLFrame):
  def __init__(self, parent, title, hid,fnMatRoi):
    HdfImageGLFrame.__init__(self, parent, title, hid)
    #HdfPyFAI1DFrame(self, title, hid)
    canvas=self.canvas
    raw=canvas.data
    
    self.prs=prs=ProcRoiStat()
    prs.SetRoiMat(fnMatRoi,raw.shape)
    prs.SetProcess('avg')
    print 'numnber of ROI,',prs.roiLenArr.size,'Total number of pixels',prs.roiIdxArr.size
    prs.Process(raw)
    print prs.resArr[0,:].max(),prs.resArr[0,:].min()
    canvas.data=prs.resArr
    #.reshape(prs.mskNumSeg,-1)
    #canvas.data=np.rot90(prs.resArr[0,:].reshape(-1,prs.mskNumSeg))

  def BuildMenu(self):
    HdfImageGLFrame.BuildMenu(self)
    mnBar=self.GetMenuBar()
    mn=mnBar.GetMenu(0)
    itemLst=mn.GetMenuItems()
    it=itemLst[0]
    it.GetItemLabel()
    #mnItem=mn.Append(wx.ID_ANY, 'Setup FAI', 'Setup fast azimutal integration ');self.Bind(wx.EVT_MENU, self.OnFAISetup, mnItem)

  @staticmethod
  def OnSetView(usrData,value,msg):
    'called when a slice is selected with the slider controls'
    frm=usrData.slider.Parent
    ds=frm.dataSet
    canvas=frm.canvas
    glImg=canvas.glImg
    sl=ut.GetSlice(frm.idxXY,ds.shape,frm.wxAxCtrlLst)
    
    prs=frm.prs
    prs.Process(ds[sl])
    #canvas.data[:]=(np.rot90(prs.resArr[0,:].reshape(-1,prs.mskNumSeg)))[:]
    canvas.data[:]=prs.resArr[:]
    glImg.data[:]=canvas.GetTxrData()
    glImg.update()
    canvas.OnPaint(None)#force to repaint, Refresh and Update do not force !
    #canvas.Refresh(False)
    #canvas.Update()
    pass

  def OnProcRoiStatSetup(self, event):
    dlg=DlgSetupProcRoiStat(self)
    if dlg.ShowModal()==wx.ID_OK:
      pass
    dlg.Destroy()


  
if __name__ == '__main__':
  import os,sys,argparse #since python 2.7
  def GetParser(required=True):
    fnHDF='/scratch/detectorData/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/scan_00106-00161.hdf5'
    lbl='pilatus_1'
    elem='/entry/data/'+lbl
    fnMatRoi='/scratch/detectorData/cSAXS_2013_10_e14608_georgiadis_3D_for_Marianne/analysis/data/pilatus_integration_mask.mat'    
    exampleCmd='--hdfFile='+fnHDF+' --elem='+elem
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog='Example:\n'+os.path.basename(sys.argv[0])+' '+exampleCmd+'\n ')
    parser.add_argument('--matRoi', required=required, default=fnMatRoi, help='the hdf5 to show')
    parser.add_argument('--hdfFile', required=required, default=fnHDF, help='the hdf5 to show')
    parser.add_argument('--elem', required=required, default=elem, help='the path to the element in the hdf5 file')
    return parser
    args = parser.parse_args()
    return args

  class App(wx.App):
    def OnInit(self):
      parser=GetParser()
      #parser=GetParser(False) # debug with exampleCmd
      args = parser.parse_args()
      try:
        self.fid=fid=h5py.h5f.open(args.hdfFile)
      except IOError as e:
        sys.stderr.write('Unable to open File: '+args.hdfFile+'\n')
        parser.print_usage(sys.stderr)
        return True
      try:
        hid = h5py.h5o.open(fid,args.elem)
      except KeyError as e:
        sys.stderr.write('Unable to open Object: '+args.elem+'\n')
        parser.print_usage(sys.stderr)
        return True
      frame = ProcRoiStatFrame(None,args.elem,hid,args.matRoi)
      frame.Show()
      self.SetTopWindow(frame)
      return True

    def OnExit(self):
      self.fid.close()

  ut.StopWatch.Start()
  app = App()
  app.MainLoop()





#!/usr/bin/env python
#*-----------------------------------------------------------------------*
#|                                                                       |
#|  Copyright (c) 2013 by Paul Scherrer Institute (http://www.psi.ch)    |
#|                                                                       |
#|              Author Thierry Zamofing (thierry.zamofing@psi.ch)        |
#*-----------------------------------------------------------------------*
'''
hdf5 tree viewer to dispay the tree of a hdf5 file.
'''

import os
import wx,h5py

class HdfTreeCtrl(wx.TreeCtrl):
  def __init__(self, parent, *args, **kwargs):
    wx.TreeCtrl.__init__(self, parent, *args, **kwargs)
    il = wx.ImageList(16, 16)
    rootDir=os.path.join(os.path.dirname(__file__),'images')
    for img in ('home','folder','dataset','compound','text','warning','link'):
      il.Add(wx.Image(rootDir+'/%s.png'%img, wx.BITMAP_TYPE_PNG).ConvertToBitmap())
    self.AssignImageList(il)

  def _ShowHirarchy(self,wxParent,gid,lvl):
    #numObj=gid.get_num_objs()
    #for idx in range(numObj):
    #  hidStr=get_objname_by_idx(idx)
    #  get_objtype_by_idx(INT idx)
    for hidStr in h5py.h5g.GroupIter(gid):
      lid=gid.links.get_info(hidStr)
      try:
        hid = h5py.h5o.open(gid,hidStr)
      except KeyError as e:
        hid=None
        t=None
      if lid.type==h5py.h5l.TYPE_EXTERNAL:
        if hid==None:
          image=5
        else:
          image=6
        hid=(hid,)+gid.links.get_val(hidStr)
        t=type(hid)
      else:
        t=type(hid)
        if t==h5py.h5g.GroupID:
          image=1
        elif t==h5py.h5d.DatasetID:
          tt=type(hid.get_type())
          if tt==h5py.h5t.TypeCompoundID:
            image=3
          elif tt==h5py.h5t.TypeStringID:
            image=4
          else:
            image=2
        else:
          image=-1
      wxNode = self.AppendItem(wxParent, hidStr,image=image,data=wx.TreeItemData(hid))
      if t==h5py.h5g.GroupID:
        self._ShowHirarchy(wxNode,hid,lvl+1)

  def ShowHirarchy(self,hid):
    self.hid=hid
    t=type(hid)
    if t==h5py.h5f.FileID:
      txt=os.path.basename(hid.name)
    elif t==h5py.h5g.GroupID:
      txt=hid.name
    else:
      txt='root'
    self.DeleteAllItems()
    wxNode = self.AddRoot(txt,image=0,data=wx.TreeItemData(hid))
    self._ShowHirarchy(wxNode,hid,0)

    #self.ExpandAll()
    self.ExpandLvl(wxNode,3)

  def ExpandLvl(self,wxNode,lvl):
    #print lvl,self.GetItemText(wxNode)
    self.Expand(wxNode)
    lvl=lvl-1
    if lvl==0:
      return
    wxCld,cookie=self.GetFirstChild(wxNode)
    while True:
      if not wxCld.IsOk():
        break
      self.ExpandLvl(wxCld,lvl)
      wxCld,cookie=self.GetNextChild(wxNode,cookie)

if __name__ == '__main__':
  import utilities as ut
  import os,sys,argparse #since python 2.7
  def GetParser(required=True):   
    fnHDF='/scratch/detectorData/e14472_00033.hdf5'
    #lbl='mcs'
    #lbl='pilatus_1'
    lbl='spec'
    exampleCmd='--hdfFile='+fnHDF
    parser = argparse.ArgumentParser(formatter_class=argparse.RawDescriptionHelpFormatter,
                                     description=__doc__,
                                     epilog='Example:\n'+os.path.basename(sys.argv[0])+' '+exampleCmd+'\n ')
    parser.add_argument('--hdfFile', required=required, default=fnHDF, help='the hdf5 to show')
    return parser
    args = parser.parse_args()
    return args

  class HdfTreeFrame(wx.Frame):
 
    def __init__(self, parent, title, fid):
      wx.Frame.__init__(self, parent, title=title, size=wx.Size(350, 450))
      wxTree = HdfTreeCtrl(self, style=wx.TR_HAS_BUTTONS)  
      self.Centre()
      wxTree.ShowHirarchy(fid)
      self.wxTree=wxTree
      self.fid=fid

  class App(wx.App):
    def OnInit(self):
      parser=GetParser()
      #parser=GetParser(False) # debug with exampleCmd
      args = parser.parse_args()
      try:
        self.fid=fid=h5py.h5f.open(args.hdfFile)
      except IOError as e:
        sys.stderr.write('Unable to open File: '+args.hdfFile+'\n')
        parser.print_usage(sys.stderr)
        return True
      frame = HdfTreeFrame(None,args.hdfFile,fid)
      frame.Show()
      self.SetTopWindow(frame)
      return True

    def OnExit(self):
      self.fid.close()

  ut.StopWatch.Start()
  app = App()
  app.MainLoop()
